

package Text::BibTeX::CVStructure;
use Text::BibTeX qw(purify_string change_case);

use Text::BibTeX::Bib;
@ISA=qw(Text::BibTeX::BibStructure);


our %known_option=(talk=>1, 
		   course=>1,
		  refreport=>{omit_month=>1},
		  emptysections=>0,
		  refreport=>{},
		  interval=>{},
		  bibitem=>{}, 
		  inter_block_mkup=>'',
		  pre_entry_mkup=>'',
		  blist_mkup=>'',
		  elist_mkup=>'',
		  url_mkup=>['',''],
		  section_1_mkup=>'',
		  section_2_mkup=>'',		  
		  section_3_mkup=>'',		  		  
		  seccounter_1_mkup=>'',
		  seccounter_2_mkup=>'',		  
		  seccounter_3_mkup=>'',		  		  
		  emptysec_1_mkup=>'',
		  emptysec_2_mkup=>'',		  
		  emptysec_3_mkup=>'',		  		  
		  tilde_mkup=>'',
		  annote_mkup=>'',
		   omit_title=>'',
		 );

my %default_option=%known_option;

sub default_option($$){
  my $self=shift;
  my $opt=shift;

#  print STDERR "default option\n";

  if ($default_option{$opt}){
    return $opt;
  } elsif ($self->known_type($opt)) {
    return {};
  } else {
    return $self->Text::BibTeX::BibStructure::default_option($opt);
  }
  

}


sub known_option($$){
  my $self=shift;
  my $opt=shift;

  return exists $known_option{$opt} || $self->SUPER::known_option($opt)||
    $self->SUPER::known_type($opt);
}


sub get_suboption($$$){
  my $self=shift;
  my $cat=shift;
  my $opt=shift;
  
  my $href;

  if ($href=$self->get_options($cat)){
    return $href->{$opt};
  } else {
    return undef;
  }
}

sub add_section($$$){
  my $self=shift;
  my $key=shift;
  my $sec=shift;

  if (!defined($self->{SECTIONS})){
    $self->{SECTIONINDEX}=1;
    $self->{SECTIONS} =  {};
  }

  $self->{SECTIONS}->{$key}=$sec;
  $sec->{INDEX}=$self->{SECTIONINDEX}++;
}

sub section{
  my $self=shift;
  my $key=shift;
  return $self->{SECTIONS}->{$key};
}

sub sections{
  shift->{SECTIONS};
};

package Text::BibTeX::CVEntry;

use Text::BibTeX::BibFormat;
use Text::BibTeX::CVSort;

@ISA=qw(Text::BibTeX::CVSort Text::BibTeX::BibFormat);


sub get_option($){
  my $self=shift;
  my $key=shift;
  my $type=$self->type;

  my $struct=$self->structure;

  return (defined($self->section) && $self->section->get_option($key)) ||
    ($struct->known_option($type) && $struct->get_suboption($self->type, $key)) || undef;
}

sub set_section($$){
  my $self=shift;
  my $section=shift;

  $self->{SECTION}=$section;
}

sub section($){
  my $self=shift;
  
  return $self->{SECTION};
}

# utilities

sub format_enddate
{
   my $self = shift;

   my @date = grep ($_, $self->get ('endmonth', 'endyear'));
   return join (' ', @date);
}

sub format_title
{
   my $self = shift;

   my $lower = $self->structure->get_options ('atitle_lower');
   my $title = $self->get ('title') || '';
   $title = Text::BibTeX::change_case ('t', $title) if $lower;
   $self->markup_string ('atitle', $title);
##   $markup->[0] . $title . $markup->[1];
}



# entry formatting 
sub format_unpublished
{
   my $self = shift;


   my $name_block = [$self->format_authors];
   my $title_block = [$self->format_title];


   ($name_block, $title_block, $self->format_date, 
    $self->get ('note'), $self->format_url);
}  

sub format_article
{
   my $self = shift;


   my $name_block = [$self->format_authors];
   my $title_block = [$self->format_title];
   my $journal_block = [[$self->markup_string('journal', $self->get ('journal')),
                         $self->format_vol_num_pages,
                         $self->format_date]];

   ($name_block, $title_block, $journal_block, $self->get ('note'), $self->format_url);
}  # format_article

sub format_chapter{
  my $self = shift;
  return ('Chapter '. $self->get('chapter'))
};

sub format_talk{
  my $self = shift;

  return ( ( $self->structure->get_suboption('talk','omit_title') || 
	     !defined($self->get('title'))) ? () : [$self->format_title],
	   
	   [ $self->get('institution') ],
 [ $self->get('repcount') > 1 ? ($self->get('repcount'). ' seminars') : () ] );
}

sub format_refreport{
  my $self = shift;

  return ( $self->structure->get_suboption('refreport','omit_title') ? () : [$self->format_title],
	   
	   [ $self->markup_string('journal', $self->get ('journal')) ],
	 [ $self->get('repcount') > 1 ? ($self->get('repcount'). ' reports') : () ] );
}


sub format_course{
  my $self = shift;

  return ( [$self->format_title],
	   
	   [  $self->get ('institution') ],
	 [ ($self->get('repcount')||0) > 1 ? ($self->get('repcount'). ' times') : () ] );
}



sub format_url{
  my $self=shift;

  my $url=$self->get('url');
  if ($url){
    my $rval= $self->markup_string('url',$url);
    $rval=~ s/%URL%/$url/;
    return $rval;
  }
  return undef;
}


sub format_interval{
  my $self=shift;

   my $name_block = [$self->format_authors];
#  print STDERR $self->section->structure->{known_options};
   my $title_block = 
#     	$self->section->structure->get_options('omit_title') ?
#	  [] :
	  [$self->format_title] ;
   my $institution_block= [ $self->get('institution') ];


   ($name_block, $title_block, $institution_block , $self->get ('note'));

}

sub format_toappear{

   my $self = shift;

   my @blocks;
   push @blocks, [$self->format_authors];
   push @blocks, [$self->format_title];
   push @blocks, [[$self->markup_string('journal', $self->get ('journal')),
		   $self->format_vol_num_pages,
		   $self->format_date]] if ($self->exists('journal'));

   push @blocks, [ $self->get ('note'), $self->format_url ];

   return @blocks;
     

  
}

sub connect_words
{
   my ($s1, $s2) = @_;

   return $s1 . ((length ($s2) < 3) ? '~' : ' ') . $s2;
}

sub format_pages
{
   my $self = shift;

   my $pages = $self->get ('pages');
   if ($pages =~ /[,+-]/)               # multiple pages?
   {
      $pages =~ s/([^-])-([^-])/$1--$2/g;
      return connect_words ("pages", $pages);
   }
   else
   {
      return connect_words ("page", $pages);
   }
}

sub format_inproceedings{
  my $self=shift;

  return ([$self->format_authors],
	  [$self->format_title],
	  [[$self->markup_string('btitle',$self->get('booktitle')),
		$self->exists('volume')?   $self->format_vol_num_pages : 
	    $self->exists('pages') ? $self->format_pages : (),
	    $self->format_date]],
	  $self->get('note'),
	  $self->format_url
	 );
}

sub format_incollection{
  my $self=shift;

  return ([$self->format_authors],
	  [$self->format_title],
	  [[$self->markup_string('btitle',$self->get('booktitle')),
	    $self->get('series'),
	    $self->exists('chapter') ? $self->format_chapter : (),
	    $self->exists('pages') ? 
                         $self->format_vol_num_pages : (),
	    $self->format_date]],
	  $self->get('organization'),
	  $self->get('note'),
	  $self->format_url
	 );
}

sub format_proceedings{
  my $self=shift;
  my $title_block = [$self->markup_string('btitle',$self->format_title),
];
  return([$self->format_editors],
	 $title_block, 
	 $self->format_date,
	 $self->get('degree'),
	 $self->get('note'),$self->format_url);

}

sub format_mastersthesis(@){
  my $self=shift;

  my $style=  $self->section->style;

  my $institution_block= [ $self->get('school') ];
  my $title_block = [$self->markup_string('btitle',$self->format_title)];
#  print STDERR "degree= ",$self->get('degree');
  return([$self->format_authors],
	 $title_block, $institution_block, 
	 ($style ne 'dated' ? $self->format_date: () ),
	 $self->get('degree'),
	 $self->get('note'),
	 $self->format_url);
}

sub format_phdthesis(@){
  format_mastersthesis(@_);
}


1;
